@@ -99,13 +99,13 @@ class CSV s r where
   -----------------------------------------------------------------------------
   -- | Turn a stream of 's' into a stream of CSV row type. An example
   -- would be parsing a ByteString stream as rows of 'MapRow' 'Text'.
-  intoCSV :: MonadResource m => CSVSettings -> GLInfConduit s m r
+  intoCSV :: (MonadThrow m) => CSVSettings -> Conduit s m r
 
   -----------------------------------------------------------------------------
   -- | Turn a stream of CSV row type back into a stream of 's'. An
   -- example would be rendering a stream of 'Row' 'ByteString' rows as
   -- 'Text'.
-  fromCSV :: MonadResource m => CSVSettings -> GInfConduit r m s
+  fromCSV :: Monad m => CSVSettings -> Conduit r m s
 
 
 
@@ -116,8 +116,8 @@ class CSV s r where
 instance CSV ByteString (Row ByteString) where
   rowToStr s !r =
     let
-      sep = B.pack [c2w (csvOutputColSep s)]
-      wrapField !f = case csvOutputQuoteChar s of
+      sep = B.pack [c2w (csvSep s)]
+      wrapField !f = case csvQuoteChar s of
         Just !x -> (x `B8.cons` escape x f) `B8.snoc` x
         _ -> f
       escape c str = B8.intercalate (B8.pack [c,c]) $ B8.split c str
@@ -132,8 +132,8 @@ instance CSV ByteString (Row ByteString) where
 instance CSV Text (Row Text) where
   rowToStr s !r =
     let
-      sep = T.pack [csvOutputColSep s]
-      wrapField !f = case csvOutputQuoteChar s of
+      sep = T.pack [csvSep s]
+      wrapField !f = case csvQuoteChar s of
         Just !x -> x `T.cons` escape x f `T.snoc` x
         _ -> f
       escape c str = T.intercalate (T.pack [c,c]) $ T.split (== c) str
@@ -147,36 +147,36 @@ instance CSV Text (Row Text) where
 -- | 'Row' instance using 'Text' based on 'ByteString' stream
 instance CSV ByteString (Row Text) where
     rowToStr s r = T.encodeUtf8 $ rowToStr s r
-    intoCSV set = intoCSV set >+> C.map (map T.decodeUtf8)
-    fromCSV set = fromCSV set >+> C.map T.encodeUtf8
+    intoCSV set = intoCSV set =$= C.map (map T.decodeUtf8)
+    fromCSV set = fromCSV set =$= C.map T.encodeUtf8
+
+
+
+-------------------------------------------------------------------------------
+-- | 'Row' instance using 'String' based on 'ByteString' stream.
+-- Please note this uses the ByteString operations underneath and has
+-- lots of unnecessary overhead. Included for convenience.
+instance CSV ByteString (Row String) where
+    rowToStr s r = rowToStr s $ map B8.pack r
+    intoCSV set = intoCSV set =$= C.map (map B8.unpack)
+    fromCSV set = C.map (map B8.pack) =$= fromCSV set
 
 
 
 -------------------------------------------------------------------------------
 fromCSVRow :: (Monad m, IsString s, CSV s r)
-           => CSVSettings -> GInfConduit r m s
-fromCSVRow set = do
-  erow <- awaitE
-  case erow of
-    Left ures -> return ures
-    Right row -> mapM_ yield [rowToStr set row, "\n"] >> fromCSVRow set
+           => CSVSettings -> Conduit r m s
+fromCSVRow set = awaitForever $ \row -> mapM_ yield [rowToStr set row, "\n"]
 
 
 -------------------------------------------------------------------------------
 intoCSVRow :: (MonadThrow m, AttoparsecInput i)
-           => Parser i (Maybe o) -> GLInfConduit i m o
-intoCSVRow p = parse >+> puller
+           => Parser i (Maybe o) -> Conduit i m o
+intoCSVRow p = parse =$= puller
   where
     parse = {-# SCC "conduitParser_p" #-} conduitParser p
-    puller = {-# SCC "puller" #-} do
-      emrow <- awaitE
-      case emrow of
-        Left ures -> return ures
-        Right (_, mrow) ->
-          case mrow of
-            Just row -> yield row >> puller
-            Nothing -> puller
-
+    puller = {-# SCC "puller" #-}
+      awaitForever $ \ (_, mrow) -> maybe (return ()) yield mrow
 
 
 -------------------------------------------------------------------------------
@@ -189,9 +189,9 @@ instance (CSV s (Row s'), Ord s', IsString s) => CSV s (MapRow s') where
 
 
 -------------------------------------------------------------------------------
-intoCSVMap :: (Ord a, MonadResource m, CSV s [a])
-           => CSVSettings -> GLInfConduit s m (MapRow a)
-intoCSVMap set = intoCSV set >+> (headers >>= converter)
+intoCSVMap :: (Ord a, MonadThrow m, CSV s [a])
+           => CSVSettings -> Conduit s m (MapRow a)
+intoCSVMap set = intoCSV set =$= (headers >>= converter)
   where
     headers = do
       mrow <- await
@@ -199,22 +199,14 @@ intoCSVMap set = intoCSV set >+> (headers >>= converter)
         Nothing -> return []
         Just [] -> headers
         Just hs -> return hs
-    converter hs = do
-      erow <- awaitE
-      case erow of
-        Left ures -> return ures
-        Right row -> yield (toMapCSV hs row) >> converter hs
+    converter hs = awaitForever $ yield . toMapCSV hs
     toMapCSV !hs !fs = M.fromList $ zip hs fs
 
 
 -------------------------------------------------------------------------------
 fromCSVMap :: (Monad m, IsString s, CSV s [a])
-           => CSVSettings -> GInfConduit (M.Map k a) m s
-fromCSVMap set = do
-  erow <- awaitE
-  case erow of
-    Left ures -> return ures
-    Right row -> push row >> fromCSVMap set
+           => CSVSettings -> Conduit (M.Map k a) m s
+fromCSVMap set = awaitForever push
   where
     push r = mapM_ yield [rowToStr set (M.elems r), "\n"]
 
@@ -225,9 +217,9 @@ fromCSVMap set = do
 --
 -- > ... =$= writeHeaders settings >> fromCSV settings $$ sinkFile "..."
 writeHeaders
-    :: (MonadResource m, CSV s (Row r), IsString s)
+    :: (Monad m, CSV s (Row r), IsString s)
     => CSVSettings
-    -> GConduit (MapRow r) m s
+    -> Conduit (MapRow r) m s
 writeHeaders set = do
   mrow <- await
   case mrow of
@@ -245,9 +237,6 @@ writeHeaders set = do
 
 -------------------------------------------------------------------------------
 -- | Read the entire contents of a CSV file into memory.
---
--- An easy way to run this function would be 'runResourceT' after
--- feeding it all the arguments.
 readCSVFile
     :: (CSV ByteString a)
     => CSVSettings
@@ -258,7 +247,6 @@ readCSVFile
 readCSVFile set fp = runResourceT $ sourceFile fp $= intoCSV set $$ C.consume
 
 
-
 -------------------------------------------------------------------------------
 -- | Write CSV data into file.
 writeCSVFile
@@ -284,9 +272,9 @@ writeCSVFile set fo fmode rows = runResourceT $ do
 -- An easy way to run this function would be 'runResourceT' after
 -- feeding it all the arguments.
 mapCSVFile
-    :: (MonadResource m, CSV ByteString a, CSV ByteString b)
+    :: (MonadResource m, MonadThrow m, CSV ByteString a, CSV ByteString b)
     => CSVSettings
-    -- ^ Settings to use both for input and output
+    -- ^ Settings to use both for both input and output
     -> (a -> [b])
     -- ^ A mapping function
     -> FilePath
@@ -311,9 +299,9 @@ mapCSVFile set f fi fo =
 --
 -- > transformCSV set (sourceFile inFile) (C.map f) (sinkFile outFile)
 transformCSV
-    :: (MonadResource m, CSV s a, CSV s' b)
+    :: (MonadThrow m, CSV s a, CSV s' b)
     => CSVSettings
-    -- ^ Settings to be used for input and output
+    -- ^ Settings to be used for both input and output
     -> Source m s
     -- ^ A raw stream data source. Ex: 'sourceFile inFile'
     -> Conduit a m b
