@@ -1,10 +1,11 @@
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings    #-}
 {-# LANGUAGE TypeSynonymInstances #-}
 
 module Data.CSV.Conduit.Types where
 
 -------------------------------------------------------------------------------
-import qualified Data.Map as M
+import           Data.Default
+import qualified Data.Map     as M
 -------------------------------------------------------------------------------
 
 
@@ -12,42 +13,35 @@ import qualified Data.Map as M
 -- | Settings for a CSV file. This library is intended to be flexible
 -- and offer a way to process the majority of text data files out
 -- there.
-data CSVSettings = CSVS
-  { 
+data CSVSettings = CSVSettings
+  {
     -- | Separator character to be used in between fields
-    csvSep :: !Char          
+    csvSep       :: !Char
 
     -- | Quote character that may sometimes be present around fields.
     -- If 'Nothing' is given, the library will never expect quotation
     -- even if it is present.
   , csvQuoteChar :: !(Maybe Char)
-  
-    -- | Quote character that should be used in the output.
-  , csvOutputQuoteChar :: !(Maybe Char)
-  
-    -- | Field separator that should be used in the output.
-  , csvOutputColSep :: !Char
   } deriving (Read, Show, Eq)
 
 
 
 -------------------------------------------------------------------------------
--- | Default settings for a CSV file. 
+-- | Default settings for a CSV file.
 --
 -- > csvSep = ','
 -- > csvQuoteChar = Just '"'
--- > csvOutputQuoteChar = Just '"'
--- > csvOutputColSep = ','
 --
 defCSVSettings :: CSVSettings
-defCSVSettings = CSVS
+defCSVSettings = CSVSettings
   { csvSep = ','
   , csvQuoteChar = Just '"'
-  , csvOutputQuoteChar = Just '"'
-  , csvOutputColSep = ','
-  } 
+  }
 
 
+instance Default CSVSettings where
+    def = defCSVSettings
+
 -------------------------------------------------------------------------------
 -- | A 'Row' is just a list of fields
 type Row a = [a]
